# setup.py
from setuptools import setup, find_packages

setup(
    name='kaalka',
    version='1.0',
    packages=find_packages(),
    install_requires=[
        'ntplib'
    ],
    # Metadata
    author='PIYUSH-MISHRA-00',
    author_email='piyushmishra.professional@gmail.com',
    description='Kaalka encryption library',
    url='https://github.com/PIYUSH-MISHRA-00/Kaalka-Encryption-Algorithm/tree/kaalka/kaalka',
)
